@@ -45,13 +45,16 @@
 		g_value_unset (&value); \
 	} while (0)
 
-#define UPDATE_FROM_INT(type, name, format) \
+#define UPDATE_FROM_INT(type, name, format, empty) \
 	do { \
 		char *temp; \
 		bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw), \
 						 type, &value); \
-		temp = g_strdup_printf (gettext (format), \
+		if (g_value_get_int (&value) != 0) \
+			temp = g_strdup_printf (gettext (format), \
 					g_value_get_int (&value)); \
+		else \
+			temp = g_strdup (empty); \
 		bacon_video_widget_properties_set_label (props, name, temp); \
 		g_free (temp); \
 		g_value_unset (&value); \
@@ -103,6 +106,7 @@ bacon_video_widget_properties_finalize (GObject *object)
 	g_return_if_fail (object != NULL);
 
 	g_object_unref (props->priv->xml);
+	g_free (props->priv);
 
 	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
@@ -146,20 +150,24 @@ bacon_video_widget_properties_reset (BaconVideoWidgetProperties *props)
 	bacon_video_widget_properties_from_time (props, 0);
 
 	/* Dimensions */
-	bacon_video_widget_properties_set_label (props, "dimensions", _("0 x 0"));
+	bacon_video_widget_properties_set_label (props, "dimensions", _("N/A"));
 	/* Video Codec */
 	bacon_video_widget_properties_set_label (props, "vcodec", _("N/A"));
 	/* Video Bitrate */
 	bacon_video_widget_properties_set_label (props, "video_bitrate",
-			_("0 kbps"));
+			_("N/A"));
 	/* Framerate */
 	bacon_video_widget_properties_set_label (props, "framerate",
-			_("0 frames per second"));
+			_("N/A"));
 	/* Audio Bitrate */
 	bacon_video_widget_properties_set_label (props, "audio_bitrate",
-			_("0 kbps"));
+			_("N/A"));
 	/* Audio Codec */
 	bacon_video_widget_properties_set_label (props, "acodec", _("N/A"));
+	/* Sample rate */
+	bacon_video_widget_properties_set_label (props, "samplerate", _("0 Hz"));
+	/* Channels */
+	bacon_video_widget_properties_set_label (props, "channels", _("0 Channels"));
 }
 
 void
@@ -221,9 +229,10 @@ bacon_video_widget_properties_update (BaconVideoWidgetProperties *props,
 				  "dimensions", N_("%d x %d"));
 		UPDATE_FROM_STRING (BVW_INFO_VIDEO_CODEC, "vcodec");
 		UPDATE_FROM_INT (BVW_INFO_FPS, "framerate",
-				 N_("%d frames per second"));
+				 N_("%d frames per second"), _("N/A"));
 		UPDATE_FROM_INT (BVW_INFO_VIDEO_BITRATE, "video_bitrate",
-				 N_("%d kbps"));
+				 N_("%d kbps"), _("N/A"));
+		gtk_widget_show (item);
 	} else {
 		gtk_widget_hide (item);
 	}
@@ -239,8 +248,11 @@ bacon_video_widget_properties_update (BaconVideoWidgetProperties *props,
 	if (has_type != FALSE)
 	{
 		UPDATE_FROM_INT (BVW_INFO_AUDIO_BITRATE, "audio_bitrate",
-				 N_("%d kbps"));
+				 N_("%d kbps"), _("N/A"));
 		UPDATE_FROM_STRING (BVW_INFO_AUDIO_CODEC, "acodec");
+		UPDATE_FROM_INT (BVW_INFO_AUDIO_SAMPLE_RATE, "samplerate",
+				N_("%d Hz"), _("N/A"));
+		UPDATE_FROM_STRING (BVW_INFO_AUDIO_CHANNELS, "channels");
 	}
 
 #undef UPDATE_FROM_STRING
@@ -283,7 +295,7 @@ bacon_video_widget_properties_new (void)
 
 	props->priv->xml = xml;
 	vbox = glade_xml_get_widget (props->priv->xml, "vbox1");
-	gtk_box_pack_start (GTK_BOX (props), vbox, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (props), vbox, FALSE, FALSE, 0);
 
 	bacon_video_widget_properties_reset (props);
 
