@@ -69,6 +69,10 @@ struct TotemPlaylistPrivate
 
 	/* This is the playing icon */
 	GdkPixbuf *icon;
+	GtkIconTheme *theme;
+
+	GtkActionGroup *action_group;
+	GtkUIManager *ui_manager;
 
 	/* This is a scratch list for when we're removing files */
 	GList *list;
@@ -107,7 +111,7 @@ enum {
 };
 
 enum {
-	PIX_COL,
+	PLAYING_COL,
 	FILENAME_COL,
 	URI_COL,
 	TITLE_CUSTOM_COL,
@@ -131,6 +135,14 @@ static const GtkTargetEntry target_table[] = {
 	{ "_NETSCAPE_URL", 0, 1 },
 };
 
+static void playlist_remove_action_activated (GtkAction *action, TotemPlaylist *playlist);
+static void playlist_copy_location_action_activated (GtkAction *action, TotemPlaylist *playlist);
+
+static const GtkActionEntry entries[] = {
+	{ "remove", GTK_STOCK_REMOVE, N_("_Remove"), NULL, N_("Remove file from playlist"), G_CALLBACK (playlist_remove_action_activated) },
+	{ "copy-location", GTK_STOCK_COPY, N_("_Copy location"), NULL, N_("Copy the location to the clipboard"), G_CALLBACK (playlist_copy_location_action_activated) }
+};
+
 static GtkWidgetClass *parent_class = NULL;
 
 static void totem_playlist_class_init (TotemPlaylistClass *class);
@@ -226,8 +238,8 @@ totem_playlist_mrl_to_title (const gchar *mrl)
 
 	filename = g_path_get_basename (mrl);
 	unescaped = gnome_vfs_unescape_string_for_display (filename);
-
 	g_free (filename);
+
 	filename_for_display = g_filename_to_utf8 (unescaped,
 			-1,             /* length */
 			NULL,           /* bytes_read */
@@ -238,8 +250,12 @@ totem_playlist_mrl_to_title (const gchar *mrl)
 	{
 		filename_for_display = g_locale_to_utf8 (unescaped,
 				-1, NULL, NULL, NULL);
-		if (filename_for_display == NULL)
-			return unescaped;
+		if (filename_for_display == NULL) {
+			filename_for_display = g_filename_display_name
+				(unescaped);
+		}
+		g_free (unescaped);
+		return filename_for_display;
 	}
 
 	g_free (unescaped);
@@ -448,7 +464,7 @@ drop_cb (GtkWidget     *widget,
 }
 
 static void
-on_copy1_activate (GtkButton *button, TotemPlaylist *playlist)
+playlist_copy_location_action_activated (GtkAction *action, TotemPlaylist *playlist)
 {
 	GList *l;
 	GtkTreePath *path;
@@ -481,35 +497,69 @@ on_copy1_activate (GtkButton *button, TotemPlaylist *playlist)
 }
 
 static gboolean
-treeview_button_pressed (GtkTreeView *treeview, GdkEventButton *event,
-		TotemPlaylist *playlist)
+playlist_show_popup_menu (TotemPlaylist *playlist, GdkEventButton *event)
 {
+	guint button = 0;
+       	guint32 time;
 	GtkTreePath *path;
+	gint count;
 	GtkWidget *menu;
+	GtkAction *action; 
 
-	if (event->type != GDK_BUTTON_PRESS
-			|| event->button != 3)
-		return FALSE;
+	if (event != NULL) {
+		button = event->button;
+		time = event->time;
 
-	if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview),
-				event->x, event->y, &path,
-				NULL, NULL, NULL) == FALSE)
-	{
+		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (playlist->_priv->treeview),
+				 event->x, event->y, &path, NULL, NULL, NULL)) {
+			if (!gtk_tree_selection_path_is_selected (playlist->_priv->selection, path)) {
+				gtk_tree_selection_unselect_all (playlist->_priv->selection);
+				gtk_tree_selection_select_path (playlist->_priv->selection, path);
+			}
+			gtk_tree_path_free (path);
+		} else {
+			gtk_tree_selection_unselect_all (playlist->_priv->selection);
+		}
+	} else {
+		time = gtk_get_current_event_time ();
+	}
+
+	count = gtk_tree_selection_count_selected_rows (playlist->_priv->selection);
+	
+	if (count == 0) {
 		return FALSE;
 	}
 
-	gtk_tree_selection_unselect_all (playlist->_priv->selection);
-	gtk_tree_selection_select_path (playlist->_priv->selection, path);
-	gtk_tree_path_free (path);
+	action = gtk_action_group_get_action (playlist->_priv->action_group, "copy-location");
+	gtk_action_set_sensitive (action, count == 1);
+
+	menu = gtk_ui_manager_get_widget (playlist->_priv->ui_manager, "/totem-playlist-popup");
 
-	menu = glade_xml_get_widget (playlist->_priv->xml, "menu1");
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
-			event->button, event->time);
 	gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
 
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
+			button, time);
+
 	return TRUE;
 }
 
+static gboolean
+treeview_button_pressed (GtkTreeView *treeview, GdkEventButton *event,
+		TotemPlaylist *playlist)
+{
+	if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
+		return playlist_show_popup_menu (playlist, event);
+	}
+
+	return FALSE;
+}
+
+static gboolean
+playlist_treeview_popup_menu (GtkTreeView *treeview, TotemPlaylist *playlist)
+{
+	return playlist_show_popup_menu (playlist, NULL);
+}
+
 static void
 totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 {
@@ -526,8 +576,8 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 	{
 		GtkTreeIter iter;
 		char *index;
-		GdkPixbuf *pixbuf;
 		GtkTreePath *path;
+		gboolean playing;
 
 		index = g_strdup_printf ("%d", i);
 		if (gtk_tree_model_get_iter_from_string
@@ -539,13 +589,10 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 		}
 		g_free (index);
 
-		gtk_tree_model_get (playlist->_priv->model, &iter,
-				PIX_COL, &pixbuf, -1);
-
-		if (pixbuf == NULL)
+		gtk_tree_model_get (playlist->_priv->model, &iter, PLAYING_COL, &playing, -1);
+		if (!playing) {
 			continue;
-
-		gdk_pixbuf_unref (pixbuf);
+		}
 
 		/* Only emit the changed signal if we changed the ->current */
 		path = gtk_tree_path_new_from_indices (i, -1);
@@ -642,6 +689,26 @@ selection_changed (GtkTreeSelection *treeselection, TotemPlaylist *playlist)
 	gtk_widget_set_sensitive (down_button, sensitivity);
 }
 
+static void
+load_icon (TotemPlaylist *playlist)
+{
+	gint w, h;
+
+	if (playlist->_priv->icon != NULL) {
+		g_object_unref (G_OBJECT (playlist->_priv->icon));
+	}
+
+	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
+	playlist->_priv->icon = gtk_icon_theme_load_icon
+		(playlist->_priv->theme, "audio-volume-medium", w, 0, NULL);
+}
+
+static void
+icon_theme_changed (GtkIconTheme *theme, TotemPlaylist *playlist)
+{
+	load_icon (playlist);
+}
+
 /* This function checks if the current item is NULL, and try to update it
  * as the first item of the playlist if so. It returns TRUE if there is a
  * current item */
@@ -714,7 +781,7 @@ totem_playlist_foreach_selected (GtkTreeModel *model, GtkTreePath *path,
 }
 
 static void
-totem_playlist_remove_files (GtkWidget *widget, TotemPlaylist *playlist)
+playlist_remove_files (TotemPlaylist *playlist)
 {
 	GtkTreeSelection *selection;
 	GtkTreeRowReference *ref;
@@ -821,6 +888,18 @@ totem_playlist_remove_files (GtkWidget *widget, TotemPlaylist *playlist)
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (playlist->_priv->treeview));
 }
 
+static void
+playlist_remove_button_clicked (GtkWidget *button, TotemPlaylist *playlist)
+{
+	playlist_remove_files (playlist);
+}
+
+static void
+playlist_remove_action_activated (GtkAction *action, TotemPlaylist *playlist)
+{
+	playlist_remove_files (playlist);
+}
+
 static void
 totem_playlist_save_playlist (TotemPlaylist *playlist, char *filename, gint active_format)
 {
@@ -832,7 +911,7 @@ totem_playlist_save_playlist (TotemPlaylist *playlist, char *filename, gint acti
 							save_types[active_format - 1].type);
 	else {
 		for (i = 0; i < G_N_ELEMENTS(save_types); i++) {
-			if(g_str_has_suffix (filename, save_types[i].suffix)) {
+			if (g_str_has_suffix (filename, save_types[i].suffix)) {
 				cur = &save_types[i];
 				break;
 			}
@@ -855,10 +934,12 @@ totem_playlist_save_add_format_combo_box (GtkFileChooser *fc)
 	gtk_widget_show (label);
 
 	combo_box = gtk_combo_box_new_text ();
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), _("By extension"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box),
+			_("By extension"));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), 0);
 	for (i = 0; i < G_N_ELEMENTS(save_types); i++) {
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box), save_types[i].name);
+		gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box),
+				save_types[i].name);
 	}
 	gtk_widget_show (combo_box);
 
@@ -867,6 +948,13 @@ totem_playlist_save_add_format_combo_box (GtkFileChooser *fc)
 	gtk_widget_show (hbox);
 	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (fc), hbox);
 
+	atk_object_add_relationship (gtk_widget_get_accessible (label),
+			ATK_RELATION_LABEL_FOR,
+			gtk_widget_get_accessible (combo_box));
+	atk_object_add_relationship (gtk_widget_get_accessible (combo_box),
+			ATK_RELATION_LABELLED_BY,
+			gtk_widget_get_accessible (label));
+
 	return combo_box;
 }
 
@@ -885,6 +973,7 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 			NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (fs), GTK_RESPONSE_ACCEPT);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
+	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (fs), TRUE);
 	/* translators: Playlist is the default saved playlist filename,
 	 * without the suffix */
 	filename = g_strconcat (_("Playlist"), save_types[0].suffix, NULL);
@@ -906,8 +995,6 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 	if (response == GTK_RESPONSE_ACCEPT)
 	{
 		char *filename;
-		GnomeVFSResult res;
-		GnomeVFSFileInfo info;
 		gint active_format;
 
 		filename = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (fs));
@@ -921,35 +1008,6 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 		g_free (playlist->_priv->save_path);
 		playlist->_priv->save_path = g_path_get_dirname (filename);
 
-		res = gnome_vfs_get_file_info (filename, &info, GNOME_VFS_FILE_INFO_DEFAULT);
-		if (res != GNOME_VFS_ERROR_NOT_FOUND && res == GNOME_VFS_OK)
-		{
-			GtkWidget *dialog;
-
-			dialog = gtk_message_dialog_new
-				(totem_playlist_get_toplevel (playlist),
-				 GTK_DIALOG_MODAL,
-				 GTK_MESSAGE_QUESTION,
-				 GTK_BUTTONS_NONE,
-				 _("Overwrite file?"));
-			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-								  _("A file named '%s' already exists.  Are you sure you want to overwrite it?"),
-								  filename);
-
-			gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-			gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-					GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
-					GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-					NULL);
-
-			response = gtk_dialog_run (GTK_DIALOG (dialog));
-			gtk_widget_destroy (dialog);
-			if (response != GTK_RESPONSE_ACCEPT)
-			{
-				g_free (filename);
-				return;
-			}
-		}
 		totem_playlist_save_playlist (playlist, filename, active_format);
 		g_free (filename);
 	} else {
@@ -1085,33 +1143,6 @@ totem_playlist_down_files (GtkWidget *widget, TotemPlaylist *playlist)
 	totem_playlist_move_files (playlist, FALSE);
 }
 
-static void
-totem_playlist_popup_menu (TotemPlaylist *playlist, guint time)
-{
-	GtkTreePath *path;
-	GtkWidget *menu;
-	int x, y;
-
-	if (gdk_window_get_pointer (playlist->_priv->treeview->window,
-			&x, &y, NULL) == NULL)
-		return;
-
-	if (gtk_tree_view_get_path_at_pos
-			(GTK_TREE_VIEW (playlist->_priv->treeview),
-			 x, y, &path, NULL, NULL, NULL) == FALSE) {
-		return;
-	}
-
-	gtk_tree_selection_unselect_all (playlist->_priv->selection);
-	gtk_tree_selection_select_path (playlist->_priv->selection, path);
-	gtk_tree_path_free (path);
-
-	menu = glade_xml_get_widget (playlist->_priv->xml, "menu1");
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
-			0, time);
-	gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
-}
-
 static int
 totem_playlist_key_press (GtkWidget *win, GdkEventKey *event, TotemPlaylist *playlist)
 {
@@ -1123,17 +1154,6 @@ totem_playlist_key_press (GtkWidget *win, GdkEventKey *event, TotemPlaylist *pla
 				(playlist->_priv->selection);
 			return TRUE;
 		}
-
-		if (event->state & GDK_SHIFT_MASK
-		    && event->keyval == GDK_F10) {
-			totem_playlist_popup_menu (playlist, event->time);
-			return TRUE;
-		}
-	}
-
-	if (event->keyval == GDK_Menu) {
-		totem_playlist_popup_menu (playlist, event->time);
-		return TRUE;
 	}
 
 	/* If we have modifiers, and either Ctrl, Mod1 (Alt), or any
@@ -1149,7 +1169,7 @@ totem_playlist_key_press (GtkWidget *win, GdkEventKey *event, TotemPlaylist *pla
 
 	if (event->keyval == GDK_Delete)
 	{
-		totem_playlist_remove_files (NULL, playlist);
+		playlist_remove_files (playlist);
 		return TRUE;
 	}
 
@@ -1157,7 +1177,26 @@ totem_playlist_key_press (GtkWidget *win, GdkEventKey *event, TotemPlaylist *pla
 }
 
 static void
-init_columns (GtkTreeView *treeview)
+set_playing_icon (GtkTreeViewColumn *column, GtkCellRenderer *renderer,
+		GtkTreeModel *model, GtkTreeIter *iter, TotemPlaylist *playlist)
+{
+	gboolean playing;
+
+	gtk_tree_model_get (model, iter, PLAYING_COL, &playing, -1);
+
+	if (playing) {
+		g_object_set (G_OBJECT (renderer),
+				"pixbuf", playlist->_priv->icon,
+				NULL);
+	} else {
+		g_object_set (G_OBJECT (renderer),
+				"pixbuf", NULL,
+				NULL);
+	}
+}
+
+static void
+init_columns (GtkTreeView *treeview, TotemPlaylist *playlist)
 {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
@@ -1166,8 +1205,8 @@ init_columns (GtkTreeView *treeview)
 	renderer = gtk_cell_renderer_pixbuf_new ();
 	column = gtk_tree_view_column_new ();
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
-	gtk_tree_view_column_set_attributes (column, renderer,
-			"pixbuf", PIX_COL, NULL);
+	gtk_tree_view_column_set_cell_data_func (column, renderer,
+			(GtkTreeCellDataFunc) set_playing_icon, playlist, NULL);
 	gtk_tree_view_append_column (treeview, column);
 
 	/* Labels */
@@ -1230,7 +1269,7 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 
 	/* the model */
 	model = GTK_TREE_MODEL (gtk_list_store_new (NUM_COLS,
-				GDK_TYPE_PIXBUF,
+				G_TYPE_BOOLEAN,
 				G_TYPE_STRING,
 				G_TYPE_STRING,
 				G_TYPE_BOOLEAN,
@@ -1244,7 +1283,7 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
 	g_object_unref (G_OBJECT (model));
 
-	init_columns (GTK_TREE_VIEW (treeview));
+	init_columns (GTK_TREE_VIEW (treeview), playlist);
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
@@ -1254,6 +1293,8 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 			G_CALLBACK (treeview_row_changed), playlist);
 	g_signal_connect (G_OBJECT (treeview), "button-press-event",
 			G_CALLBACK (treeview_button_pressed), playlist);
+	g_signal_connect (G_OBJECT (treeview), "popup-menu",
+			G_CALLBACK (playlist_treeview_popup_menu), playlist);
 
 	/* Drag'n'Drop */
 	g_signal_connect (G_OBJECT (treeview), "drag_data_received",
@@ -1264,7 +1305,7 @@ init_treeview (GtkWidget *treeview, TotemPlaylist *playlist)
 			G_CALLBACK (button_release_cb), playlist);
 	g_signal_connect (G_OBJECT (treeview), "drag_begin",
                         G_CALLBACK (drag_begin_cb), playlist);
-         g_signal_connect (G_OBJECT (treeview), "drag_end",
+	g_signal_connect (G_OBJECT (treeview), "drag_end",
                         G_CALLBACK (drag_end_cb), playlist);
 	gtk_drag_dest_set (treeview, GTK_DEST_DEFAULT_ALL,
 			target_table, G_N_ELEMENTS (target_table),
@@ -1433,6 +1474,25 @@ totem_playlist_entry_parsed (TotemPlParser *parser,
 	totem_playlist_add_one_mrl (playlist, uri, title);
 }
 
+static void
+totem_playlist_realize (GtkWidget *widget, TotemPlaylist *playlist)
+{
+	playlist->_priv->theme = gtk_icon_theme_get_for_screen
+		(gtk_widget_get_screen (GTK_WIDGET (widget)));
+	g_signal_connect (G_OBJECT (playlist->_priv->theme), "changed",
+			G_CALLBACK (icon_theme_changed), playlist);
+	load_icon (playlist);
+}
+
+static void
+totem_playlist_unrealize (GtkWidget *widget, TotemPlaylist *playlist)
+{
+	if (playlist->_priv->theme != NULL) {
+		g_object_unref (playlist->_priv->theme);
+		playlist->_priv->theme = NULL;
+	}
+}
+
 static void
 totem_playlist_init (TotemPlaylist *playlist)
 {
@@ -1448,6 +1508,15 @@ totem_playlist_init (TotemPlaylist *playlist)
 			"entry",
 			G_CALLBACK (totem_playlist_entry_parsed),
 			playlist);
+
+	g_signal_connect (G_OBJECT (playlist),
+			"realize",
+			G_CALLBACK (totem_playlist_realize),
+			playlist);
+	g_signal_connect (G_OBJECT (playlist),
+			"unrealize",
+			G_CALLBACK (totem_playlist_unrealize),
+			playlist);
 }
 
 static void
@@ -1459,10 +1528,20 @@ totem_playlist_finalize (GObject *object)
 
 	if (playlist->_priv->current != NULL)
 		gtk_tree_path_free (playlist->_priv->current);
-	if (playlist->_priv->icon != NULL)
-		gdk_pixbuf_unref (playlist->_priv->icon);
+	if (playlist->_priv->icon != NULL) {
+		g_object_unref (playlist->_priv->icon);
+		playlist->_priv->icon = NULL;
+	}
 	g_object_unref (playlist->_priv->parser);
 
+	if (playlist->_priv->ui_manager != NULL) {
+		g_object_unref (G_OBJECT (playlist->_priv->ui_manager));
+	}
+
+	if (playlist->_priv->action_group != NULL) {
+		g_object_unref (G_OBJECT (playlist->_priv->action_group));
+	}
+
 	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 	}
@@ -1473,6 +1552,7 @@ totem_playlist_new (void)
 {
 	TotemPlaylist *playlist;
 	GtkWidget *container, *item;
+	char *filename;
 
 	playlist = TOTEM_PLAYLIST (g_object_new (GTK_TYPE_PLAYLIST, NULL));
 
@@ -1485,6 +1565,25 @@ totem_playlist_new (void)
 		return NULL;
 	}
 
+	/* popup menu */
+	playlist->_priv->action_group = 
+		gtk_action_group_new ("playlist-action-group");
+	gtk_action_group_set_translation_domain (playlist->_priv->action_group,
+			GETTEXT_PACKAGE);
+	gtk_action_group_add_actions (playlist->_priv->action_group, entries,
+			G_N_ELEMENTS (entries), playlist);
+
+	playlist->_priv->ui_manager = gtk_ui_manager_new ();
+	gtk_ui_manager_insert_action_group (playlist->_priv->ui_manager,
+			playlist->_priv->action_group, -1);
+
+	filename = totem_interface_get_full_path ("playlist-ui.xml");
+	if (!gtk_ui_manager_add_ui_from_file (playlist->_priv->ui_manager, filename, NULL)) {
+		totem_playlist_finalize (G_OBJECT (playlist));
+		return NULL;
+	}
+	g_free (filename);
+
 	/* Connect the buttons */
 	item = glade_xml_get_widget (playlist->_priv->xml, "add_button");
 	g_signal_connect (G_OBJECT (item), "clicked",
@@ -1492,7 +1591,7 @@ totem_playlist_new (void)
 			playlist);
 	item = glade_xml_get_widget (playlist->_priv->xml, "remove_button");
 	g_signal_connect (G_OBJECT (item), "clicked",
-			G_CALLBACK (totem_playlist_remove_files),
+			G_CALLBACK (playlist_remove_button_clicked),
 			playlist);
 	item = glade_xml_get_widget (playlist->_priv->xml, "save_button");
 	g_signal_connect (G_OBJECT (item), "clicked",
@@ -1507,10 +1606,6 @@ totem_playlist_new (void)
 			G_CALLBACK (totem_playlist_down_files),
 			playlist);
 
-	item = glade_xml_get_widget (playlist->_priv->xml, "copy1");
-	g_signal_connect (G_OBJECT (item), "activate",
-			G_CALLBACK (on_copy1_activate), playlist);
-
 	gtk_widget_add_events (GTK_WIDGET (playlist), GDK_KEY_PRESS_MASK);
 	g_signal_connect (G_OBJECT (playlist), "key_press_event",
 			G_CALLBACK (totem_playlist_key_press), playlist);
@@ -1536,9 +1631,6 @@ totem_playlist_new (void)
 	/* The configuration */
 	init_config (playlist);
 
-	playlist->_priv->icon =
-		totem_interface_load_pixbuf ("playlist-playing.png");
-
 	gtk_widget_show_all (GTK_WIDGET (playlist));
 
 	return GTK_WIDGET (playlist);
@@ -1569,7 +1661,7 @@ totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl,
 
 	store = GTK_LIST_STORE (playlist->_priv->model);
 	gtk_list_store_insert_with_values (store, &iter, G_MAXINT32,
-			PIX_COL, NULL,
+			PLAYING_COL, FALSE,
 			FILENAME_COL, filename_for_display,
 			URI_COL, uri,
 			TITLE_CUSTOM_COL, display_name ? TRUE : FALSE,
@@ -1896,7 +1988,7 @@ totem_playlist_has_next_mrl (TotemPlaylist *playlist)
 }
 
 gboolean
-totem_playlist_set_title (TotemPlaylist *playlist, const char *title)
+totem_playlist_set_title (TotemPlaylist *playlist, const char *title, gboolean force)
 {
 	GtkListStore *store;
 	GtkTreeIter iter;
@@ -1915,11 +2007,13 @@ totem_playlist_set_title (TotemPlaylist *playlist, const char *title)
 	if (&iter == NULL)
 		return FALSE;
 
-	gtk_tree_model_get (playlist->_priv->model, &iter,
-			TITLE_CUSTOM_COL, &custom_title,
-			-1);
-	if (custom_title != FALSE)
-		return TRUE;
+	if (force == FALSE) {
+		gtk_tree_model_get (playlist->_priv->model, &iter,
+				TITLE_CUSTOM_COL, &custom_title,
+				-1);
+		if (custom_title != FALSE)
+			return TRUE;
+	}
 
 	gtk_list_store_set (store, &iter,
 			FILENAME_COL, title,
@@ -1951,14 +2045,9 @@ totem_playlist_set_playing (TotemPlaylist *playlist, gboolean state)
 
 	g_return_val_if_fail (&iter != NULL, FALSE);
 
-	if (state != FALSE)
-		gtk_list_store_set (store, &iter,
-				PIX_COL, playlist->_priv->icon,
-				-1);
-	else
-		gtk_list_store_set (store, &iter,
-				PIX_COL, NULL,
-				-1);
+	gtk_list_store_set (store, &iter,
+			PLAYING_COL, state,
+			-1);
 
 	if (state == FALSE)
 		return TRUE;
